<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:template match="/">
    <xsl:variable name="component_name">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="nerv_icde/component/name" />
            <xsl:with-param name="replace" select="' '" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
cmake_minimum_required(VERSION 2.6)
project("py_nerv<!--xsl:value-of select="$component_name"/-->")

SET(CMAKE_BUILD_TYPE MinSizeRel)

# Python Setting
FIND_PACKAGE(PythonInterp 3 REQUIRED)
if (PYTHONINTERP_FOUND)
    MESSAGE(STATUS "PYTHONINTERP_FOUND = python ${PYTHON_VERSION_MAJOR}${PYTHON_VERSION_MINOR}")
    # Automatic Setting
    if (PYTHON_VERSION_MAJOR EQUAL 3)
        FIND_PACKAGE(PythonInterp 3)
        FIND_PACKAGE(PythonLibs 3 REQUIRED)
        SET( BOOST_PYTHON "python-py${PYTHON_VERSION_MAJOR}${PYTHON_VERSION_MINOR}")
        SET( BOOST_PYTHON_3m "python${PYTHON_VERSION_MAJOR}${PYTHON_VERSION_MINOR}")
        SET( BOOST_PYTHON_3 "python${PYTHON_VERSION_MAJOR}")
        MESSAGE(STATUS "BOOST_PYTHON = ${BOOST_PYTHON}")
    else()
        FIND_PACKAGE(PythonInterp)
        FIND_PACKAGE(PythonLibs REQUIRED)
        SET( BOOST_PYTHON "python")
    endif()
else()
    MESSAGE(STATUS "PYTHONINTERP_FOUND = 0")
    # Python Manual Setting
    SET(Python_VERSION_MAJOR 3)
    SET(Python_VERSION_MINOR 7)
    if(MSVC)
        SET(PYTHON_INCLUDE_DIRS "C:/Program Files/Python38/include")
        SET(PYTHON_LIBRARY_DIRS "C:/Program Files/Python38/libs")
        SET(PYTHON_LIBRARIES "python3.8")
    else()
        SET(PYTHON_INCLUDE_DIRS "/usr/include/python3.7m")
        # SET(PYTHON_LIBRARY_DIRS "/usr/lib/x86_64-linux-gnu") # default
        SET(PYTHON_LIBRARIES "python3.7m")
    endif()
    SET( BOOST_PYTHON "python-${Python_VERSION_MAJOR}${Python_VERSION_MINOR}")
endif()
MESSAGE(STATUS "PYTHON_INCLUDE_DIRS = ${PYTHON_INCLUDE_DIRS}")
MESSAGE(STATUS "PYTHON_LIBRARY_DIRS = ${PYTHON_LIBRARY_DIRS}")
MESSAGE(STATUS "PYTHON_LIBRARIES = ${PYTHON_LIBRARIES}")
MESSAGE(STATUS "PYTHON_EXECUTABLE = ${PYTHON_EXECUTABLE}")
if(PYTHON_INCLUDE_DIRS)
    INCLUDE_DIRECTORIES("${PYTHON_INCLUDE_DIRS}")
endif()
if(PYTHON_LIBRARY_DIRS)
    LINK_DIRECTORIES("${PYTHON_LIBRARY_DIRS}")
endif()


# Boost Python Setting
set(Boost_USE_STATIC_LIBS OFF)
set(Boost_USE_MULTITHREADED ON)
set(Boost_USE_STATIC_RUNTIME OFF)
MESSAGE(STATUS "Boost Find = ${BOOST_PYTHON}")
FIND_PACKAGE(Boost COMPONENTS ${BOOST_PYTHON} )
if(NOT Boost_FOUND)
    MESSAGE(STATUS "Boost Find = ${BOOST_PYTHON_PY}")
    FIND_PACKAGE(Boost COMPONENTS ${BOOST_PYTHON_3m} )
endif()
if(NOT Boost_FOUND)
    MESSAGE(STATUS "Boost Find = ${BOOST_PYTHON_3}")
    FIND_PACKAGE(Boost COMPONENTS ${BOOST_PYTHON_3} )
endif()
if(NOT Boost_FOUND)
    MESSAGE(STATUS "Boost Find Failed")
    SET(Boost_INCLUDE_DIRS "$ENV{Boost_INCLUDE_DIR}")
    SET(Boost_LIBRARY_DIRS "$ENV{Boost_LIBRARY_DIR}")
    SET(Boost_LIBRARIES "") # Automatic link(import from Source code )
endif()

MESSAGE(STATUS "Boost_INCLUDE_DIRS = ${Boost_INCLUDE_DIRS}")
MESSAGE(STATUS "Boost_LIBRARY_DIRS = ${Boost_LIBRARY_DIRS}")
MESSAGE(STATUS "Boost_LIBRARIES = ${Boost_LIBRARIES}")
if(Boost_INCLUDE_DIRS)
    INCLUDE_DIRECTORIES("${Boost_INCLUDE_DIRS}")
endif()
if(Boost_LIBRARY_DIRS)
    LINK_DIRECTORIES("${Boost_LIBRARY_DIRS}")
endif()

# Nerv Setting
if(MSVC)
    SET(Nerv_INCLUDE_DIRS "C:/Program Files/Waregram/Nerv/usr/include")
    SET(Nerv_LIBRARY_DIRS "C:/Program Files/Waregram/Nerv/usr/lib/x64")
    SET(Nerv_LIBRARIES "nerv_sdk_4")
else()
    # SET(Nerv_INCLUDE_DIR "/usr/include") # default
    # SET(Nerv_LIBRARY_DIR "/usr/lib") # default
    # SET(Nerv_LIBRARIES "'Nerv_General_SDK' 'Nerv_General_Common'")
    SET(Nerv_LIBRARIES "Nerv_General_SDK")
endif()
MESSAGE(STATUS "Nerv_INCLUDE_DIRS = ${Nerv_INCLUDE_DIRS}")
MESSAGE(STATUS "Nerv_LIBRARY_DIRS = ${Nerv_LIBRARY_DIRS}")
MESSAGE(STATUS "Nerv_LIBRARIES = ${Nerv_LIBRARIES}")
if(Nerv_INCLUDE_DIRS)
    INCLUDE_DIRECTORIES("${Nerv_INCLUDE_DIRS}")
endif()
if(Nerv_LIBRARY_DIRS)
    LINK_DIRECTORIES("${Nerv_LIBRARY_DIRS}")
endif()


# Etc

LINK_DIRECTORIES(/usr/lib)

INCLUDE_DIRECTORIES(../src/include)
INCLUDE_DIRECTORIES(${CMAKE_SOURCE_DIR})

# set library path environment variable
if (MSVC)
    ADD_DEFINITIONS(-D_USRDLL)
    add_compile_options("/utf-8")
    add_compile_options("/std:c++17")
    add_compile_options("/bigobj")
    add_compile_options("/wd4267")
else()
    ADD_DEFINITIONS(-DLINUX)
    add_compile_options("-std=c++17")
    add_compile_options("-fPIC")
endif()

ADD_DEFINITIONS(-DCPU_64)
ADD_DEFINITIONS(-DPY_NERV)

# set output directory
SET(CMAKE_ARCHIVE_OUTPUT_DIRECTORY ${CMAKE_SOURCE_DIR}/../..)
SET(CMAKE_LIBRARY_OUTPUT_DIRECTORY ${CMAKE_SOURCE_DIR}/../..)
SET(CMAKE_RUNTIME_OUTPUT_DIRECTORY ${CMAKE_SOURCE_DIR}/../..)

# output setting
# ADD_EXECUTABLE (${PROJECT_NAME} ${CMAKE_SOURCE_DIR}/../src/py_nerv.cpp
ADD_LIBRARY(${PROJECT_NAME} SHARED ${CMAKE_SOURCE_DIR}/../src/py_nerv.cpp
<xsl:for-each select="/nerv_icde/component/proxy_list/row">${CMAKE_SOURCE_DIR}/../src/Nerv/Proxy/<xsl:value-of select="class_name"/>/<xsl:value-of select="class_name"/>.cpp </xsl:for-each>
<xsl:for-each select="/nerv_icde/component/stub_list/row">${CMAKE_SOURCE_DIR}/../src/Nerv/Stub/<xsl:value-of select="class_name"/>/<xsl:value-of select="class_name"/>.cpp </xsl:for-each>)

# link library
TARGET_LINK_LIBRARIES(${PROJECT_NAME} ${PYTHON_LIBRARIES})
TARGET_LINK_LIBRARIES(${PROJECT_NAME} ${Boost_LIBRARIES})
TARGET_LINK_LIBRARIES(${PROJECT_NAME} ${Nerv_LIBRARIES})

if(MSVC)
    SET_TARGET_PROPERTIES(${PROJECT_NAME} PROPERTIES SUFFIX ".pyd")
else()
    SET_TARGET_PROPERTIES(${PROJECT_NAME} PROPERTIES PREFIX "")
endif()

</xsl:template>

</xsl:stylesheet>
