<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" omit-xml-declaration="yes" indent="no"/>
<xsl:param name = "export_data_domain_path" />
<xsl:param name = "export_data_name" />
<xsl:template match="/">
  <xsl:for-each select="nerv_icde/datadomain/row">
    <xsl:if test="path=$export_data_domain_path">
      <xsl:variable name="open_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' { namespace '" />
        </xsl:call-template></xsl:variable>
      <xsl:variable name="close_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' */ } /* '" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="define_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="'__'" />
        </xsl:call-template>
      </xsl:variable><xsl:text/>
/******************************************************************************
    domain   : <xsl:value-of select="name"/>
    domain creator   : <xsl:value-of select="author"/>
    domain create date : <xsl:value-of select="create-day"/>
    domain description : <xsl:value-of select="description"/>
      <xsl:for-each select="data/row">
        <xsl:if test="name=$export_data_name">
    data name : <xsl:value-of select="name"/>
    data writer   : <xsl:value-of select="../../author"/>
    data description: <xsl:value-of select="description"/>
*******************************************************************************/<xsl:text/>

#pragma once<xsl:text/>

#include &lt;Python.h&gt;<xsl:text/>
#include &lt;boost/python.hpp&gt;<xsl:text/>
#include &lt;Nerv/Nerv.h&gt;<xsl:text/>
#include &lt;Nerv/Data/sizeof.h&gt;<xsl:text/>

          <xsl:if test="type='alias'">
            <xsl:variable name="real_domain_id" select="real_domain_id"/>
            <xsl:variable name="real_data" select="real_data"/>
            <xsl:for-each select="/nerv_icde/datadomain/row">
              <xsl:if test="id=$real_domain_id">
#include &lt;Nerv/Data<xsl:value-of select="path"/>/<xsl:value-of select="$real_data"/>.h&gt;<xsl:text/>
              </xsl:if>
            </xsl:for-each>
          </xsl:if>
          <xsl:if test="type='struct'">
            <xsl:for-each select="datafield/row">
              <xsl:variable name="real_domain_id" select="real_domain_id"/>
              <xsl:variable name="real_data" select="real_data"/>
              <xsl:if test="type='alias'">
                <xsl:for-each select="/nerv_icde/datadomain/row">
                  <xsl:if test="id=$real_domain_id">
#include &lt;Nerv/Data<xsl:value-of select="path"/>/<xsl:value-of select="$real_data"/>.h&gt;<xsl:text/>
                  </xsl:if>
                </xsl:for-each>
              </xsl:if>
            </xsl:for-each>
          </xsl:if>

namespace Nerv { namespace Data <xsl:value-of select="$open_namespace_path" /> {<xsl:text>
</xsl:text>
          <xsl:if test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
inline const double min_<xsl:value-of select="name"/>() { return double(<xsl:value-of select="scmin"/>); }
inline const double max_<xsl:value-of select="name"/>() { return double(<xsl:value-of select="scmax"/>); }
          </xsl:if>

          <xsl:choose>
            <xsl:when test="type='scaled64_t'">
typedef ::Nerv::real&lt; ::Nerv::int64_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='scaled32_t'">
typedef ::Nerv::real&lt; ::Nerv::int32_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='scaled16_t'">
typedef ::Nerv::real&lt; ::Nerv::int16_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='scaled8_t'">
typedef ::Nerv::real&lt; ::Nerv::int8_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_scaled64_t'">
typedef ::Nerv::real&lt; ::Nerv::u_int64_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_scaled32_t'">
typedef ::Nerv::real&lt; ::Nerv::u_int32_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_scaled16_t'">
typedef ::Nerv::real&lt; ::Nerv::u_int16_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_scaled8_t'">
typedef ::Nerv::real&lt; ::Nerv::u_int8_t, min_<xsl:value-of select="name"/>, max_<xsl:value-of select="name"/> &gt; <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='enum64_t'">
typedef ::Nerv::enum64_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='enum32_t'">
typedef ::Nerv::enum32_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='enum16_t'">
typedef ::Nerv::enum16_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='enum8_t'">
typedef ::Nerv::enum8_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='int64_t'">
typedef ::Nerv::int64_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='int32_t'">
typedef ::Nerv::int32_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='int16_t'">
typedef ::Nerv::int16_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='int8_t'">
typedef ::Nerv::int8_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_int64_t'">
typedef ::Nerv::u_int64_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_int32_t'">
typedef ::Nerv::u_int32_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_int16_t'">
typedef ::Nerv::u_int16_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='u_int8_t'">
typedef ::Nerv::u_int8_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='ieee754_32_t'">
typedef ::Nerv::ieee754_32_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='ieee754_64_t'">
typedef ::Nerv::ieee754_64_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='char_t'">
typedef ::Nerv::char_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='wchar_t'">
typedef ::Nerv::wchar_t <xsl:value-of select="name"/>;
            </xsl:when>
            <xsl:when test="type='alias'">
              <xsl:variable name="real_data_domain_namespace">
                <xsl:call-template name="get_data_domain_namespace">
                  <xsl:with-param name="data_domain_id" select="real_domain_id" />
                </xsl:call-template>
              </xsl:variable>
typedef ::Nerv::Data<xsl:value-of select="$real_data_domain_namespace"/>::<xsl:value-of select="real_data"/><xsl:text> </xsl:text><xsl:value-of select="name"/>;
            </xsl:when>
          </xsl:choose>
          <xsl:if test="type='enum64_t' or type='enum32_t' or type='enum16_t' or type='enum8_t'">
            <xsl:for-each select="datafield/row">
const <xsl:value-of select="../../name"/><xsl:text disable-output-escaping="yes"> </xsl:text><xsl:value-of select="member_name"/> = <xsl:value-of select="scmin"/>;
            </xsl:for-each>
          </xsl:if>
          <xsl:if test="type!='struct'">
typedef template_history_elem&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history_elem;
typedef template_history&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history;
          </xsl:if>
          <xsl:if test="type='struct'">
            <xsl:variable name="maxDynamicSize">
              <xsl:for-each select="datafield/row">
                <xsl:variable name="real_data_domain_namespace">
                  <xsl:call-template name="get_data_domain_namespace">
                    <xsl:with-param name="data_domain_id" select="real_domain_id" />
                  </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="member_type">
                  <xsl:choose>
                    <xsl:when test="type='u_scaled64_t'">::Nerv::real&lt; ::Nerv::u_int64_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='u_scaled32_t'">::Nerv::real&lt; ::Nerv::u_int32_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='u_scaled16_t'">::Nerv::real&lt; ::Nerv::u_int16_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='u_scaled8_t'">::Nerv::real&lt; ::Nerv::u_int8_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='scaled64_t'">::Nerv::real&lt; ::Nerv::int64_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='scaled32_t'">::Nerv::real&lt; ::Nerv::int32_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='scaled16_t'">::Nerv::real&lt; ::Nerv::int16_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='scaled8_t'">::Nerv::real&lt; ::Nerv::int8_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                    <xsl:when test="type='alias'"><xsl:value-of select="$real_data_domain_namespace"/>::<xsl:value-of select="real_data"/></xsl:when>
                    <xsl:when test="type='int8_t'">::Nerv::int8_t</xsl:when>
                    <xsl:when test="type='int16_t'">::Nerv::int16_t</xsl:when>
                    <xsl:when test="type='int32_t'">::Nerv::int32_t</xsl:when>
                    <xsl:when test="type='int64_t'">::Nerv::int64_t</xsl:when>
                    <xsl:when test="type='u_int8_t'">::Nerv::u_int8_t</xsl:when>
                    <xsl:when test="type='u_int16_t'">::Nerv::u_int16_t</xsl:when>
                    <xsl:when test="type='u_int32_t'">::Nerv::u_int32_t</xsl:when>
                    <xsl:when test="type='u_int64_t'">::Nerv::u_int64_t</xsl:when>
                    <xsl:when test="type='ieee754_32_t'">::Nerv::ieee754_32_t</xsl:when>
                    <xsl:when test="type='ieee754_64_t'">::Nerv::ieee754_64_t</xsl:when>
                    <xsl:when test="type='char_t'">::Nerv::char_t</xsl:when>
                    <xsl:when test="type='wchar_t'">::Nerv::wchar_t</xsl:when>
                    <xsl:otherwise>::Nerv::<xsl:value-of select="type"/></xsl:otherwise>
                  </xsl:choose>
                </xsl:variable>
                <xsl:if test="arraytype='VARIABLE'">+sizeof(<xsl:value-of select="$member_type"/>)*<xsl:value-of select="arraysize"/></xsl:if>
              </xsl:for-each>
            </xsl:variable>
#pragma pack(1)
#pragma warning(disable: 4200)
struct __attribute__((packed)) <xsl:value-of select="name"/>
{<xsl:text/>
            <xsl:for-each select="datafield/row">
              <xsl:variable name="description_aligned">
                <xsl:call-template name="replace">
                  <xsl:with-param name="text" select="description" />
                  <xsl:with-param name="replace" select="'&#10;'" />
                  <xsl:with-param name="by" select="'&#10;    // '" />
                </xsl:call-template>
              </xsl:variable>
              <xsl:variable name="real_data_domain_namespace">
                <xsl:call-template name="get_data_domain_namespace">
                  <xsl:with-param name="data_domain_id" select="real_domain_id" />
                </xsl:call-template>
              </xsl:variable>
              <xsl:variable name="member_type">
                <xsl:choose>
                  <xsl:when test="type='u_scaled64_t'">::Nerv::real&lt; ::Nerv::u_int64_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='u_scaled32_t'">::Nerv::real&lt; ::Nerv::u_int32_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='u_scaled16_t'">::Nerv::real&lt; ::Nerv::u_int16_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='u_scaled8_t'">::Nerv::real&lt; ::Nerv::u_int8_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='scaled64_t'">::Nerv::real&lt; ::Nerv::int64_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='scaled32_t'">::Nerv::real&lt; ::Nerv::int32_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='scaled16_t'">::Nerv::real&lt; ::Nerv::int16_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='scaled8_t'">::Nerv::real&lt; ::Nerv::int8_t, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                  <xsl:when test="type='alias'"><xsl:value-of select="$real_data_domain_namespace"/>::<xsl:value-of select="real_data"/></xsl:when>
                  <xsl:when test="type='int8_t'">::Nerv::int8_t</xsl:when>
                  <xsl:when test="type='int16_t'">::Nerv::int16_t</xsl:when>
                  <xsl:when test="type='int32_t'">::Nerv::int32_t</xsl:when>
                  <xsl:when test="type='int64_t'">::Nerv::int64_t</xsl:when>
                  <xsl:when test="type='u_int8_t'">::Nerv::u_int8_t</xsl:when>
                  <xsl:when test="type='u_int16_t'">::Nerv::u_int16_t</xsl:when>
                  <xsl:when test="type='u_int32_t'">::Nerv::u_int32_t</xsl:when>
                  <xsl:when test="type='u_int64_t'">::Nerv::u_int64_t</xsl:when>
                  <xsl:when test="type='ieee754_32_t'">::Nerv::ieee754_32_t</xsl:when>
                  <xsl:when test="type='ieee754_64_t'">::Nerv::ieee754_64_t</xsl:when>
                  <xsl:when test="type='char_t'">::Nerv::char_t</xsl:when>
                  <xsl:when test="type='wchar_t'">::Nerv::wchar_t</xsl:when>
                  <xsl:otherwise>::Nerv::<xsl:value-of select="type"/></xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
<xsl:text>
    </xsl:text>// <xsl:value-of select="$description_aligned"/><xsl:text/>
              <xsl:if test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
<xsl:text>
    </xsl:text>inline static const double min_<xsl:value-of select="member_name"/>() { return double(<xsl:value-of select="scmin"/>); }<xsl:text/>
    inline static const double max_<xsl:value-of select="member_name"/>() { return double(<xsl:value-of select="scmax"/>); }<xsl:text/>
              </xsl:if>
              <xsl:if test="arraytype='NO'">
<xsl:text>
    </xsl:text><xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/><xsl:if test="not(bitsize = '0')"> :<xsl:value-of select="bitsize"/></xsl:if>;<xsl:text/>
                <xsl:if test="(type='alias') and (bitsize = '0')">
<xsl:text>
    </xsl:text><xsl:value-of select="$member_type"/><xsl:text>&amp; </xsl:text><xsl:value-of select="member_name"/>_ref() { return <xsl:value-of select="member_name"/>; }<xsl:text/>
                </xsl:if>
              </xsl:if>
              <xsl:if test="arraytype='FIX'">
<xsl:text>
    </xsl:text><xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/>[<xsl:if test="arraysize='0'">1</xsl:if><xsl:if test="arraysize!='0'"><xsl:value-of select="arraysize"/></xsl:if>];<xsl:text/>
                <xsl:choose>
                  <xsl:when test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
<xsl:text>
    </xsl:text>inline double <xsl:value-of select="member_name"/>_get_at(size_t index) { return (double)<xsl:value-of select="member_name"/>[index]; }<xsl:text/>
<xsl:text>
    </xsl:text>inline void <xsl:value-of select="member_name"/>_set_at(size_t index, double in_val) { <xsl:value-of select="member_name"/>[index] = in_val; }<xsl:text/>
                  </xsl:when>
                  <xsl:when test="type='alias'">
          <xsl:value-of select="$member_type"/>&amp;<xsl:text> </xsl:text><xsl:value-of select="member_name"/>_at(size_t index=0) { return <xsl:value-of select="member_name"/>[index]; }<xsl:text/>
                  </xsl:when>
                  <xsl:otherwise>
<xsl:text>
    </xsl:text>inline <xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/>_get_at(size_t index) { return <xsl:value-of select="member_name"/>[index]; }<xsl:text/>
<xsl:text>
    </xsl:text>inline void <xsl:value-of select="member_name"/>_set_at(size_t index, <xsl:value-of select="$member_type"/> in_val) { <xsl:value-of select="member_name"/>[index] = in_val; }<xsl:text/>
                  </xsl:otherwise>
                </xsl:choose>
<xsl:text>
    </xsl:text>inline PyObject* <xsl:value-of select="member_name"/>_memory(int in_write = 0) { return (PyObject*)PyMemoryView_FromMemory( (char*)<xsl:value-of select="member_name"/> , <xsl:value-of select="arraysize"/> * sizeof( <xsl:value-of select="$member_type"/>) , in_write? PyBUF_WRITE : PyBUF_READ ); }<xsl:text/>
<xsl:text>
    </xsl:text>inline int <xsl:value-of select="member_name"/>_maxsize() { return <xsl:value-of select="arraysize"/>; }<xsl:text/>
              </xsl:if>
              <xsl:if test="arraytype='VARIABLE'">
<xsl:text>
    </xsl:text>variable_array_field_dctrl_t <xsl:value-of select="member_name"/>_dctrl;<xsl:text/>
                <xsl:choose>
                  <xsl:when test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
<xsl:text>
    </xsl:text>inline double <xsl:value-of select="member_name"/>_get_at(size_t index) { return (double)((<xsl:value-of select="$member_type"/>*)(dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex))[index]; }<xsl:text/>
<xsl:text>
    </xsl:text>inline void <xsl:value-of select="member_name"/>_set_at(size_t index, double in_val) { ((<xsl:value-of select="$member_type"/>*)(dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex))[index] = in_val; }<xsl:text/>
                  </xsl:when>
                  <xsl:when test="type='alias'">
<xsl:text>
    </xsl:text><xsl:value-of select="$member_type"/>&amp; <xsl:value-of select="member_name"/>_at(size_t index=0) { return ((<xsl:value-of select="$member_type"/>*)(dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex))[index]; }<xsl:text/>
                  </xsl:when>
                  <xsl:otherwise>
<xsl:text>
    </xsl:text>inline <xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/>_get_at(size_t index) { return ((<xsl:value-of select="$member_type"/>*)(dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex))[index]; }<xsl:text/>
<xsl:text>
    </xsl:text>inline void <xsl:value-of select="member_name"/>_set_at(size_t index, <xsl:value-of select="$member_type"/> in_val) { ((<xsl:value-of select="$member_type"/>*)(dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex))[index] = in_val; }<xsl:text/>
                  </xsl:otherwise>
                </xsl:choose>
<xsl:text>
    </xsl:text>inline PyObject* <xsl:value-of select="member_name"/>_memory(int in_write = 0) { return (PyObject*)PyMemoryView_FromMemory( (char*)dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex , <xsl:value-of select="member_name"/>_size() * sizeof( <xsl:value-of select="$member_type"/>), in_write? PyBUF_WRITE : PyBUF_READ ); }<xsl:text/>
<xsl:text>
    </xsl:text>inline int <xsl:value-of select="member_name"/>_maxsize() { return <xsl:value-of select="arraysize"/>; }<xsl:text/>
<xsl:text>
    </xsl:text><xsl:value-of select="$member_type"/>* <xsl:value-of select="member_name"/>() { return (<xsl:value-of select="$member_type"/>*)(dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex); }<xsl:text/>
<xsl:text>
    </xsl:text><xsl:value-of select="$member_type"/> const* <xsl:value-of select="member_name"/>() const { return (<xsl:value-of select="$member_type"/> const*)(dynamicbuffer+<xsl:value-of select="member_name"/>_dctrl.dindex); }<xsl:text/>
<xsl:text>
    </xsl:text>bool <xsl:value-of select="member_name"/>_alloc(int array_size)
    {
        bool ret_b;
        <xsl:value-of select="member_name"/>_dctrl.dindex = dynamicsize;
        <xsl:value-of select="member_name"/>_dctrl.dsize = array_size*sizeof(<xsl:value-of select="$member_type"/>);
        if(_maxDynamicSize &lt; dynamicsize + <xsl:value-of select="member_name"/>_dctrl.dsize) {
            ret_b=false;
        }
        else {
            dynamicsize += <xsl:value-of select="member_name"/>_dctrl.dsize;
            ret_b=true;
        }
        return ret_b;
    }<xsl:text/>
<xsl:text>
    </xsl:text>int <xsl:value-of select="member_name"/>_size() const
    {
        return <xsl:value-of select="member_name"/>_dctrl.dsize / sizeof(<xsl:value-of select="$member_type"/>);
    }
              </xsl:if>
<xsl:text>
</xsl:text>
            </xsl:for-each>
            <xsl:if test="$maxDynamicSize!=''">
    /* 가변 데이터 버퍼 */
    enum {_maxDynamicSize=0<xsl:value-of select="$maxDynamicSize"/> };
    void init_dynamic_buffer() { dynamicsize = 0; }
    ::Nerv::pksize_t get_sizeof() const { return sizeof(<xsl:value-of select="name"/>)- _maxDynamicSize + dynamicsize; }
    ::Nerv::pksize_t dynamicsize;
    ::Nerv::u_int8_t dynamicbuffer[_maxDynamicSize];  // dynamic buffer

    <xsl:value-of select="name"/>()
    {
        init_dynamic_buffer();
    }

            </xsl:if>
            <xsl:if test="$maxDynamicSize=''">
    ::Nerv::pksize_t get_sizeof() const { return sizeof(*this); }
            </xsl:if>

    void copy_from(<xsl:value-of select="name"/>* _src_ )
    {
        memcpy(this, _src_, _src_->get_sizeof() );
    }
};
#pragma warning(default : 4200)
#pragma pack()
inline ::Nerv::pksize_t nerv_sizeof( <xsl:value-of select="name"/> const* const&amp; p)
{
    return p->get_sizeof();
}
typedef template_history_elem&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history_elem;
typedef template_history&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history;

          </xsl:if>
} } /* <xsl:value-of select="$close_namespace_path" /> */
        </xsl:if>
      </xsl:for-each>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


</xsl:stylesheet>
