<?xml version="1.0" encoding="UTF-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_method_domain_path" />

<xsl:template match="/">
<xsl:for-each select="nerv_icde/methoddomain/row">
<xsl:if test="path=$export_method_domain_path">
<xsl:variable name="open_namespace_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="' { namespace '" />
</xsl:call-template>
</xsl:variable>
<xsl:variable name="close_namespace_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="' */ } /* '" />
</xsl:call-template>
</xsl:variable>
<xsl:variable name="define_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="'__'" />
</xsl:call-template>
</xsl:variable>
/****************************************************************************************************************************************************
    Document Header
    domain   : <xsl:value-of select="name"/>
    creator  : <xsl:value-of select="author"/>
    create date          : <xsl:value-of select="create-day"/>
    description          : <xsl:value-of select="description"/>
****************************************************************************************************************************************************/

#pragma once

/* Include Nerv Sdk Document ***********************************************************************************************************************/
#include &lt;Nerv/Nerv.h&gt;
#include &lt;boost/python.hpp&gt;
#include &lt;stdio.h&gt;

/* Include Data Domains Document *****************************************************************************************************************/
<xsl:for-each select="ref_data_domain/row">
    <xsl:variable name="ref_domain_id" select="data_domain_id"/>
    <xsl:for-each select="/nerv_icde/datadomain/row">
        <xsl:if test="id=$ref_domain_id">
#include &lt;Nerv/Data<xsl:value-of select="path"/>/<xsl:value-of select="name"/>.h&gt;
        </xsl:if>
    </xsl:for-each>
</xsl:for-each>

/* Domain Define ***********************************************************************************************************************************/

namespace Nerv { namespace Method <xsl:value-of select="$open_namespace_path" /> {

    using namespace Nerv;

    /* Standard Domain Port Constance **************************************************************************************************************/
    const ::Nerv::domain_t c_standard_port = <xsl:value-of select="standard_port"/>;


    /* Method Defines ******************************************************************************************************************************/

<xsl:for-each select="method/row">
    <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    /********************************************************************************************************************************************
        Method       : <xsl:value-of select="name"/>
        Input        : <xsl:if test="in_data != '(null)'">Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/></xsl:if>
        output       : <xsl:if test="out_data != '(null)'">Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/></xsl:if>
        Method Type  : <xsl:value-of select="type"/>
        Code         : <xsl:value-of select="code"/>
        Writer       : <xsl:value-of select="../../author"/>
        Description  : <xsl:value-of select="../../description"/>
    ********************************************************************************************************************************************/
  <xsl:if test="type='INFORMATION'" ><xsl:call-template name="template_information" /></xsl:if>
  <xsl:if test="type='COMMAND'" ><xsl:call-template name="template_command" /></xsl:if>
  <xsl:if test="type='CALL'" ><xsl:call-template name="template_call" /></xsl:if>
  <xsl:if test="type='SIGNAL'" ><xsl:call-template name="template_signal" /></xsl:if>
  <xsl:if test="type='EVENT'" ><xsl:call-template name="template_event" /></xsl:if>
  <xsl:if test="type='BROADCAST'" ><xsl:call-template name="template_broadcast" /></xsl:if>
</xsl:for-each>


    /* Interface Defines ***************************************************************************************************************************/


<xsl:for-each select="interface/row">
    /********************************************************************************************************************************************
        Interface Name : <xsl:value-of select="name"/>
        Writer : <xsl:value-of select="../../author"/>
        Description : <xsl:value-of select="description"/>
    *******************************************************************************************************************************************/

    /* proxy */
    class proxy_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv::nerv_proxy_object<xsl:for-each select="membermethod/row">
    <xsl:variable name="method_name" select="method_name"/>
    <xsl:for-each select="../../../../method/row[name=$method_name and type!='CALLBACK' and type!='SIGNALBACK']">
    , virtual public proxy_method_<xsl:value-of select="$method_name"/>
    </xsl:for-each>
    </xsl:for-each>
    {
    };

    /* stub */
    class stub_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv::nerv_object<xsl:for-each select="membermethod/row">
    <xsl:variable name="method_name" select="method_name"/>
    <xsl:for-each select="../../../../method/row[name=$method_name and type!='CALLBACK' and type!='SIGNALBACK']">
    , virtual public method_<xsl:value-of select="$method_name"/>
    </xsl:for-each>
    </xsl:for-each>
    {
    };
</xsl:for-each>

} } /* <xsl:value-of select="$close_namespace_path" /> */


</xsl:if>
</xsl:for-each>
</xsl:template>


<xsl:template name ="template_information" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_information
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_information(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        //
        //
        //
        //
        //
        //
        //
        //

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>_connect( bool _udp_request = true, double _limit = 1.0, double* _elapsed_ = 0)
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            return_object_t return_object = async_connect( _udp_request , _limit );
            ret = async_connect_return( return_object, _elapsed_ );
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_shutdown()
        {
            Py_BEGIN_ALLOW_THREADS
            shutdown();
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_set_sync(bool _use_on_sync)
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync(_use_on_sync);
            Py_END_ALLOW_THREADS
        }

        letter_case_t <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            letter_case_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history(_size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size(size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size( _history_size);
            Py_END_ALLOW_THREADS
        }

        <xsl:if test="out_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>_read( Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            nerv_err_t ret;
            Nerv::pksize_t inf_size = sizeof(Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);
            Py_BEGIN_ALLOW_THREADS
            ret = read(_inf_dat_, &amp;inf_size );
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>

    private:

        //
        //
        //
        //

        virtual void on_disconnected()
        {
            on_<xsl:value-of select="name"/>_disconnected();
        }

        virtual void on_sync( const Nerv::sender_sync_model_t&amp; _sync_model,
                              const double&amp; _elapsed,
                              const double&amp; _limit,
                              void const* const _inf_dat_,
                              const pksize_t _inf_size)
        {
            on_<xsl:value-of select="name"/>_sync(_sync_model, _elapsed, _limit <xsl:if test="out_data!='(null)'">, (const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_inf_dat_ </xsl:if>);
        }

        virtual void on_<xsl:value-of select="name"/>_disconnected() = 0;

        virtual void on_<xsl:value-of select="name"/>_sync( Nerv::sender_sync_model_t _sync_model,
                                               double _elapsed,
                                               double _limit<xsl:if test="out_data!='(null)'">,
                                               const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_ </xsl:if>) = 0;

        //
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_information
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : nerv_object_information(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_set_sync_model( Nerv::sender_sync_model_t _sync_model, double _interval)
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync_model(_sync_model,_interval);
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect( client_addr_t _client_addr )
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect( _client_addr );
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_all_client_object()
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_all_client_object();
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_node( struct_node_addr _node_addr)
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_one_client_node( _node_addr);
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_object( client_addr_t _client_addr)
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_one_client_object( _client_addr);
            Py_END_ALLOW_THREADS
        }

        <xsl:if test="out_data!='(null)'">
        Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* <xsl:value-of select="name"/>_get()
        {
            Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* ret_;
            Py_BEGIN_ALLOW_THREADS
            ret_ = (Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*) get(sizeof(Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>) );
            Py_END_ALLOW_THREADS
            return ret_;
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            Py_BEGIN_ALLOW_THREADS
            cancel();
            Py_END_ALLOW_THREADS
        }

        nerv_err_t <xsl:value-of select="name"/>_post(const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            nerv_err_t ret;
            ::Nerv::pksize_t inf_size = nerv_sizeof(_inf_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = post( inf_size);
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>

        <xsl:if test="out_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>_write(const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            nerv_err_t ret;
            ::Nerv::pksize_t inf_size = nerv_sizeof(_inf_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = write( _inf_dat_, inf_size);
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        nerv_err_t <xsl:value-of select="name"/>_write()
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = write( 0, 0);
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>

        letter_case_t <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            letter_case_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history( _size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size( size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size( _history_size);
            Py_END_ALLOW_THREADS
        }

        <xsl:if test="out_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>_read( Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            nerv_err_t ret;
            Nerv::pksize_t inf_size = sizeof(Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);
            Py_BEGIN_ALLOW_THREADS
            ret = read(_inf_dat_, &amp;inf_size );
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>

    private:

        virtual nerv_err_t on_connect( const client_addr_t&amp; _client_addr,
                                       const bool&amp; _tcp_request,
                                       const double&amp; _elapsed,
                                       const double&amp; _limit )
        {
            return on_<xsl:value-of select="name"/>_connect( _client_addr, _tcp_request, _elapsed, _limit );
        }

        virtual void on_shutdown( client_addr_t _client_addr)
        {
            on_<xsl:value-of select="name"/>_shutdown(_client_addr);
        }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_connect( client_addr_t _client_addr,
                                                                     bool _tcp_request,
                                                                     double _elapsed,
                                                                     double _limit ) = 0;

        virtual void on_<xsl:value-of select="name"/>_shutdown( client_addr_t _client_addr) = 0;
    };
</xsl:template>

<xsl:template name ="template_command" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_command
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_command(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        //
        //
        //
        //
        //
        //
        //
        //

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>_connect( double _limit = 1.0, double* _elapsed_ = 0)
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            return_object_t return_object = async_connect( _limit );
            ret = async_connect_return( return_object, _elapsed_ );
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_shutdown()
        {
            Py_BEGIN_ALLOW_THREADS
            shutdown();
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_set_sync_model( bool _using_udp,
                                                           Nerv::sender_sync_model_t _sync_model,
                                                           double _interval )
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync_model( _using_udp, _sync_model, _interval);
            Py_END_ALLOW_THREADS
        }

        <xsl:if test="in_data!='(null)'">

        Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* <xsl:value-of select="name"/>_get()
        {
            Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* ret_;
            Py_BEGIN_ALLOW_THREADS
            ret_ = (Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)get(sizeof(Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>));
            Py_END_ALLOW_THREADS
            return ret_;
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            Py_BEGIN_ALLOW_THREADS
            cancel();
            Py_END_ALLOW_THREADS
        }

        nerv_err_t <xsl:value-of select="name"/>_post( const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            nerv_err_t ret;
            ::Nerv::pksize_t cmd_size = nerv_sizeof(_cmd_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = post( cmd_size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        nerv_err_t <xsl:value-of select="name"/>_write( const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            nerv_err_t ret;
            ::Nerv::pksize_t cmd_size = nerv_sizeof(_cmd_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = write( _cmd_dat_, cmd_size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        nerv_err_t <xsl:value-of select="name"/>_read( Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            nerv_err_t ret;
            Nerv::pksize_t cmd_size = sizeof(Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>);
            Py_BEGIN_ALLOW_THREADS
            ret = read(_cmd_dat_, &amp;cmd_size );
            Py_END_ALLOW_THREADS
            return ret;
        }

        </xsl:if>
        <xsl:if test="in_data='(null)'">

        nerv_err_t <xsl:value-of select="name"/>_write()
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = write( 0, 0);
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>

        letter_case_t <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            letter_case_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history( _size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size( size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size( _history_size);
            Py_END_ALLOW_THREADS
        }


    private:

        //
        //
        //
        //

        virtual void on_disconnected()
        {
            on_<xsl:value-of select="name"/>_disconnected();
        }

        virtual void on_<xsl:value-of select="name"/>_disconnected() = 0;

        //
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_command
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : nerv_object_command(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_disconnect( client_addr_t _client_addr )
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect( _client_addr );
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_all_client_object()
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_all_client_object();
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_node( struct_node_addr _node_addr)
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_one_client_node( _node_addr);
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_object( client_addr_t _client_addr)
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_one_client_object( _client_addr);
            Py_END_ALLOW_THREADS
        }

        ////////////////////////////////////////////////////////////////////////
        void <xsl:value-of select="name"/>_set_sync(bool _use_on_sync)
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync(_use_on_sync);
            Py_END_ALLOW_THREADS
        }

        letter_case_t <xsl:value-of select="name"/>_see_history( client_addr_t _client_addr, size_t _size)
        {
            letter_case_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history(_client_addr, _size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size( size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size( _history_size);
            Py_END_ALLOW_THREADS
        }

        int <xsl:value-of select="name"/>_get_client_count()
        {
            int ret;
            Py_BEGIN_ALLOW_THREADS
            ret = get_client_count();
            Py_END_ALLOW_THREADS
            return ret;
        }
        <xsl:if test="in_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>_read(client_addr_t _client, Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            nerv_err_t ret;
            Nerv::pksize_t cmd_size = sizeof(Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>);
            Py_BEGIN_ALLOW_THREADS
            ret = read(_client, _cmd_dat_, &amp;cmd_size );
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>
    private:

        virtual void on_sync( const client_addr_t&amp; _client,
                              const Nerv::sender_sync_model_t&amp; _sync_model,
                              const double&amp; _elapsed,
                              const double&amp; _limit,
                              void const* const _cmd_dat_,
                              const pksize_t _cmd_size)
        {
            <xsl:if test="in_data!='(null)'">
            on_<xsl:value-of select="name"/>_sync(_client, _sync_model,_elapsed, _limit,(const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cmd_dat_);
            </xsl:if>
            <xsl:if test="in_data='(null)'">
            on_<xsl:value-of select="name"/>_sync(_client, _sync_model,_elapsed, _limit);
            </xsl:if>
        }

        virtual nerv_err_t on_connect( const client_addr_t&amp; _client_addr,
                                       const double&amp; _elapsed,
                                       const double&amp; _limit )
        {
            return on_<xsl:value-of select="name"/>_connect( _client_addr, _elapsed, _limit );
        }

        virtual void on_shutdown( const client_addr_t&amp; _client_addr)
        {
            on_<xsl:value-of select="name"/>_shutdown(_client_addr);
        }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_connect( client_addr_t _client_addr,
                                                                     double _elapsed,
                                                                     double _limit ) = 0;

        virtual void on_<xsl:value-of select="name"/>_shutdown( client_addr_t _client_addr) = 0;

        virtual void on_<xsl:value-of select="name"/>_sync( client_addr_t _client,
                                                   Nerv::sender_sync_model_t _sync_model,
                                                   double _elapsed,
                                                   double _limit<xsl:if test="in_data!='(null)'">,
                                                   const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_</xsl:if> ) = 0; //{}
    };
</xsl:template>

<xsl:template name ="template_call" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_call
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_call(<xsl:value-of select="code"/>) {}

        //
        //
        //
        //
        //
        //
        //
        //
        //

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>(<xsl:if test="in_data!='(null)'"> const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_,</xsl:if><xsl:if test="out_data!='(null)'"> Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_,</xsl:if> double _limit = 1.0, double* _elapsed_ = 0)
        {
            nerv_err_t ret;
            <xsl:if test="in_data!='(null)'">::Nerv::pksize_t cal_size = nerv_sizeof(_cal_dat_);</xsl:if>
            <xsl:if test="out_data!='(null)'">::Nerv::pksize_t ret_size = sizeof(Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            return_object_t return_object = async_call(<xsl:choose><xsl:when test="in_data!='(null)'">_cal_dat_,cal_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit);
            ret = async_return( return_object, <xsl:choose><xsl:when test="out_data!='(null)'">_ret_dat_, &amp;ret_size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> _elapsed_ );
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_cancel(nerv_err_t _error)
        {
            Py_BEGIN_ALLOW_THREADS
            cancel(_error);
            Py_END_ALLOW_THREADS
        }

    private:

        //
        //
        //
        //
        //
        //
        //
        //

        //
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_call
    {
    public:

        method_<xsl:value-of select="name"/>() : nerv_object_call(<xsl:value-of select="code"/>, <xsl:if test="out_data!='(null)'">sizeof(Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>)</xsl:if><xsl:if test="out_data='(null)'">0</xsl:if> ) {}

    protected:
        virtual ::Nerv::nerv_err_t on_call ( const client_addr_t&amp;  _client_addr,
                                 void* const   _cal_dat_,
                                 const pksize_t&amp;  _cal_size,
                                 void*  _ret_dat_,
                                 pksize_t*  _ret_size_,
                                 const double&amp;  _elapsed,
                                 const double&amp;  _limit )
        {
            ::Nerv::nerv_err_t r = on_<xsl:value-of select="name"/>(_client_addr,<xsl:if test="in_data!='(null)'"> (const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cal_dat_,</xsl:if><xsl:if test="out_data!='(null)'"> (Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_ret_dat_,</xsl:if> _elapsed, _limit);
  <xsl:if test="out_data!='(null)'">
            if( ! r )
            {
                *_ret_size_ = nerv_sizeof((Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_ret_dat_);
            }
  </xsl:if>
            return r;
        }

        virtual void on_call_cancel( const client_addr_t&amp; _client_addr, const nerv_err_t&amp; _error)
        {
            on_<xsl:value-of select="name"/>_cancel(_client_addr, _error);
        }

        virtual ::Nerv::nerv_err_t on_<xsl:value-of select="name"/>(::Nerv::client_addr_t client_addr, <xsl:if test="in_data!='(null)'">const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_,</xsl:if> <xsl:if test="out_data!='(null)'">Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_,</xsl:if> double _elapsed, double _limit) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel(::Nerv::client_addr_t _client_addr, ::Nerv::nerv_err_t _error) = 0;
    };
</xsl:template>

<xsl:template name ="template_signal" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_signal
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_signal(<xsl:value-of select="code"/>) {}

        //
        //
        //
        //
        //
        //
        //
        //
        //

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>(<xsl:if test="in_data!='(null)'">const Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_dat_,</xsl:if> double _limit = 1.0, double* _elapsed_= 0)
        {
            nerv_err_t ret;
            <xsl:if test="in_data!='(null)'">::Nerv::pksize_t sig_size = nerv_sizeof(_sig_dat_);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            ret = signal(<xsl:choose><xsl:when test="in_data!='(null)'">_sig_dat_, sig_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit, _elapsed_);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_cancel(nerv_err_t _error)
        {
            Py_BEGIN_ALLOW_THREADS
            cancel(_error);
            Py_END_ALLOW_THREADS
        }

    private:

        //
        //
        //
        //

        //
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_signal
    {
    public:
        method_<xsl:value-of select="name"/>() : nerv_object_signal(<xsl:value-of select="code"/>) {}

    protected:
        virtual void on_signal( const ::Nerv::client_addr_t&amp;   _client_addr,
                    void* const   _sig_dat_,
                    const ::Nerv::pksize_t&amp;   _sig_size,
                    const double&amp;   _elapsed)
        {
            on_<xsl:value-of select="name"/>(_client_addr, <xsl:if test="in_data!='(null)'">(const ::Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_sig_dat_,</xsl:if> _elapsed);
        }

        virtual void on_signal_cancel( const ::Nerv::client_addr_t&amp; _client_addr, const ::Nerv::nerv_err_t&amp; _error)
        {
            on_<xsl:value-of select="name"/>_cancel(_client_addr,_error);
        }

        virtual void on_<xsl:value-of select="name"/>(::Nerv::client_addr_t client, <xsl:if test="in_data!='(null)'">const ::Nerv::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_dat_,</xsl:if> double _elapsed) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel( ::Nerv::client_addr_t _client_addr, ::Nerv::nerv_err_t _error) = 0;
    };
</xsl:template>

<xsl:template name ="template_event" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_event
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_event(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        //
        //
        //
        //
        //
        //
        //
        //

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>_connect( double _limit = 1.0, double* _elapsed_= 0 )
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            return_object_t return_object = async_connect( _limit );
            ret = async_connect_return( return_object, _elapsed_ );
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_shutdown()
        {
            Py_BEGIN_ALLOW_THREADS
            shutdown();
            Py_END_ALLOW_THREADS
        }

    private:

        //
        //
        //
        //

        virtual void on_event( const void* const _evt_dat_, const pksize_t&amp; _event_size, const double&amp;  _elapsed_)
        {
            on_<xsl:value-of select="name"/>(<xsl:if test="out_data!='(null)'">(const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_evt_dat_,</xsl:if> _elapsed_);
        }

        virtual void on_disconnected() { on_<xsl:value-of select="name"/>_disconnected(); }

        virtual void on_<xsl:value-of select="name"/>( <xsl:if test="out_data!='(null)'">const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _evt_dat_,</xsl:if> double _elapsed) = 0; //{}

        virtual void on_<xsl:value-of select="name"/>_disconnected() = 0; //{}

        ///
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_event
    {
    public:
        method_<xsl:value-of select="name"/>() : nerv_object_event(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_disconnect(const ::Nerv::client_addr_t client_addr )
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect(client_addr);
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_all_client_object()
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_all_client_object();
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_node( struct_node_addr _node_addr)
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_one_client_node( _node_addr);
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_object( client_addr_t _client_addr)
        {
            Py_BEGIN_ALLOW_THREADS
            disconnect_one_client_object( _client_addr);
            Py_END_ALLOW_THREADS
        }

  <xsl:if test="out_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>(const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _evt_dat_)
        {
            nerv_err_t ret;
            ::Nerv::pksize_t evt_size = nerv_sizeof(_evt_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = event(_evt_dat_,evt_size);
            Py_END_ALLOW_THREADS
            return ret;
        }
  </xsl:if>
  <xsl:if test="out_data='(null)'">
        nerv_err_t <xsl:value-of select="name"/>()
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = event(0,0);
            Py_END_ALLOW_THREADS
            return ret;
        }
  </xsl:if>

    private:

        virtual ::Nerv::nerv_err_t on_connect(  const client_addr_t&amp; _client_addr,
                                                    const double&amp; _elapsed,
                                                    const double&amp; _limit )
        {
            return on_<xsl:value-of select="name"/>_connect(_client_addr, _elapsed, _limit);
        }

        virtual void on_shutdown( const ::Nerv::client_addr_t&amp; client_addr) { on_<xsl:value-of select="name"/>_shutdown( client_addr); }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_connect( client_addr_t _client_addr,
                                                      double _elapsed,
                                                      double _limit ) = 0;

        virtual void on_<xsl:value-of select="name"/>_shutdown(::Nerv::client_addr_t client_addr) = 0;
    };
</xsl:template>

<xsl:template name ="template_broadcast" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_broadcast
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_broadcast(<xsl:value-of select="code"/>)
        {
        }

    private:
        <xsl:if test="out_date='(null)'">
        virtual void on_broadcast( const struct_server_addr&amp; _server_addr, const void* _bst_dat_, const pksize_t&amp; _size, const double&amp;  _elapsed)
        {
            on_<xsl:value-of select="name"/>_broadcast(_server_addr,(const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_bst_dat_, _elapsed);
        }
        virtual void on_<xsl:value-of select="name"/>_broadcast( ::Nerv::server_addr_t _server_addr, const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _bst_dat_, double _elapsed) = 0;
        </xsl:if>
        <xsl:if test="out_date!='(null)'">
        virtual void on_broadcast( const struct_server_addr&amp; _server_addr, const void* _bst_dat_, const pksize_t&amp; _size, const double&amp;  _elapsed)
        {
            on_<xsl:value-of select="name"/>_broadcast(_server_addr, _elapsed);
        }
        virtual void on_<xsl:value-of select="name"/>_broadcast( ::Nerv::server_addr_t _server_addr, double _elapsed) = 0;
        </xsl:if>
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_broadcast
    {
    public:
        method_<xsl:value-of select="name"/>() : nerv_object_broadcast(<xsl:value-of select="code"/>)
        {
        }
        <xsl:if test="out_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>_broadcast(const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _bst_dat_)
        {
            nerv_err_t ret;
            ::Nerv::pksize_t bst_size = nerv_sizeof(_bst_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = broadcast(_bst_dat_,bst_size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        nerv_err_t <xsl:value-of select="name"/>_cast(::Nerv::ip_t _ip, const Nerv::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _bst_dat_)
        {
            nerv_err_t ret;
            ::Nerv::pksize_t bst_size = nerv_sizeof(_bst_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = broadcast(_ip, _bst_dat_,bst_size);
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        nerv_err_t <xsl:value-of select="name"/>_broadcast()
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = broadcast(0,0);
            Py_END_ALLOW_THREADS
            return ret;
        }

        nerv_err_t <xsl:value-of select="name"/>_cast(::Nerv::ip_t _ip)
        {
            nerv_err_t ret;
            Py_BEGIN_ALLOW_THREADS
            ret = broadcast(_ip, 0,0);
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>
    };
</xsl:template>

</xsl:stylesheet>
